@@ -608,12 +608,12 @@ XLALBinaryPulsarDeltaT( BinaryPulsarOutput   *output,
           e*e)*cu - x*sw*su)/(1.0 - e*cu));
       }
       else{
-        /* dt += (x*sin(w)*(cos(u)-e) + (x*cos(w)*sqrt(1.0-e*e) +
-          lal_gamma)*sin(u))*(1.0 - (LAL_TWOPI/Pb)*(x*cos(w)*sqrt(1.0 -
-          e*e)*cos(u) - x*sin(w)*sin(u))/(1.0 - e*cos(u))); */
-        dt += (x*sw*(cu-e) + (x*cw*sqrt(1.0-e*e) +
-          lal_gamma)*su)*(1.0 - (LAL_TWOPI/Pb)*(x*cw*sqrt(1.0 -
-          e*e)*cu - x*sw*su)/(1.0 - e*cu));
+          /* dt += (x*sin(w)*(cos(u)-e) + (x*cos(w)*sqrt(1.0-e*e) +
+            lal_gamma)*sin(u))*(1.0 - (LAL_TWOPI/Pb)*(x*cos(w)*sqrt(1.0 -
+            e*e)*cos(u) - x*sin(w)*sin(u))/(1.0 - e*cos(u))); */
+          dt += (x*sw*(cu-e) + (x*cw*sqrt(1.0-e*e) +
+            lal_gamma)*su)*(1.0 - (LAL_TWOPI/Pb)*(x*cw*sqrt(1.0 -
+            e*e)*cu - x*sw*su)/(1.0 - e*cu));
       }
     /**********************************************************/
     }
@@ -650,6 +650,7 @@ XLALBinaryPulsarDeltaT( BinaryPulsarOutput   *output,
     REAL8 DS, DA; /* Shapiro delay and Abberation delay terms */
     REAL8 Dbb;
     REAL8 DAOP, DSR; /* Kopeikin delay terms */
+    REAL8 fac = 1.;  /* factor in front of fb coefficients */
 
     KopeikinTerms kt;
     REAL8 Ck, Sk;
@@ -677,7 +678,22 @@ XLALBinaryPulsarDeltaT( BinaryPulsarOutput   *output,
 
     tt0 = tb - Tasc;
 
-    orbits = tt0/Pb - 0.5*(pbdot+xpbdot)*(tt0/Pb)*(tt0/Pb);
+    /* handle higher orbital frequency derivatives if present */
+    if( params->nfb > 0 ){
+      /* set period from first orbital frequency value */
+      Pb = 1./params->fb[0];
+      orbits = tt0/Pb;
+
+      fac = 1.;
+      for ( UINT4 j=1 ; j < (UINT4)params->nfb; j++){
+        fac /= (REAL8)(j+1);
+        orbits += fac*params->fb[j]*pow(tt0,j+1);
+      }
+    }
+    else{
+      orbits = tt0/Pb - 0.5*(pbdot+xpbdot)*(tt0/Pb)*(tt0/Pb);
+    }
+
     norbits = (INT4)orbits;
     if(orbits < 0.0) norbits--;
 
@@ -919,7 +935,7 @@ XLALBinaryPulsarDeltaTNew( BinaryPulsarOutput   *output,
   REAL8 m2;
   const REAL8 c3 = (REAL8)LAL_C_SI*(REAL8)LAL_C_SI*(REAL8)LAL_C_SI;
 
-  CHAR *model = PulsarGetStringParam(params, "BINARY");
+  const CHAR *model = PulsarGetStringParam(params, "BINARY");
 
   /* Check input arguments */
   if( input == (BinaryPulsarInput *)NULL ){
@@ -1052,7 +1068,7 @@ XLALBinaryPulsarDeltaTNew( BinaryPulsarOutput   *output,
         w = w0 + wdot*tt0; /* calculate w */
 
         if( !strcmp(model, "BTX") && PulsarCheckParam(params, "FB") ){
-          REAL8Vector *fb = PulsarGetREAL8VectorParam(params, "FB");
+          const REAL8Vector *fb = PulsarGetREAL8VectorParam(params, "FB");
 
           fac = 1.;
           for ( j=1 ; j < (INT4)fb->length + 1; j++){
@@ -1089,7 +1105,7 @@ XLALBinaryPulsarDeltaTNew( BinaryPulsarOutput   *output,
       if( !strcmp(model, "BTX") ){
         REAL8 fb0 = 0.;
         if ( PulsarCheckParam(params, "FB") ){
-          REAL8Vector *fb = PulsarGetREAL8VectorParam(params, "FB");
+          const REAL8Vector *fb = PulsarGetREAL8VectorParam(params, "FB");
           fb0 = fb->data[0];
         }
         /* dt += (x*sin(w)*(cos(u)-e) + (x*cos(w)*sqrt(1.0-e*e) +
@@ -1100,12 +1116,12 @@ XLALBinaryPulsarDeltaTNew( BinaryPulsarOutput   *output,
           e*e)*cu - x*sw*su)/(1.0 - e*cu));
       }
       else{
-        /* dt += (x*sin(w)*(cos(u)-e) + (x*cos(w)*sqrt(1.0-e*e) +
-          lal_gamma)*sin(u))*(1.0 - (LAL_TWOPI/Pb)*(x*cos(w)*sqrt(1.0 -
-          e*e)*cos(u) - x*sin(w)*sin(u))/(1.0 - e*cos(u))); */
-        dt += (x*sw*(cu-e) + (x*cw*sqrt(1.0-e*e) +
-          lal_gamma)*su)*(1.0 - (LAL_TWOPI/Pb)*(x*cw*sqrt(1.0 -
-          e*e)*cu - x*sw*su)/(1.0 - e*cu));
+          /* dt += (x*sin(w)*(cos(u)-e) + (x*cos(w)*sqrt(1.0-e*e) +
+            lal_gamma)*sin(u))*(1.0 - (LAL_TWOPI/Pb)*(x*cos(w)*sqrt(1.0 -
+            e*e)*cos(u) - x*sin(w)*sin(u))/(1.0 - e*cos(u))); */
+          dt += (x*sw*(cu-e) + (x*cw*sqrt(1.0-e*e) +
+            lal_gamma)*su)*(1.0 - (LAL_TWOPI/Pb)*(x*cw*sqrt(1.0 -
+            e*e)*cu - x*sw*su)/(1.0 - e*cu));
       }
     /**********************************************************/
     }
@@ -1142,6 +1158,7 @@ XLALBinaryPulsarDeltaTNew( BinaryPulsarOutput   *output,
     REAL8 DS, DA; /* Shapiro delay and Abberation delay terms */
     REAL8 Dbb;
     REAL8 DAOP, DSR; /* Kopeikin delay terms */
+    REAL8 fac = 1.;  /* factor in front of fb coefficients */
 
     KopeikinTerms kt;
     REAL8 Ck, Sk;
@@ -1169,7 +1186,24 @@ XLALBinaryPulsarDeltaTNew( BinaryPulsarOutput   *output,
 
     tt0 = tb - Tasc;
 
-    orbits = tt0/Pb - 0.5*(pbdot+xpbdot)*(tt0/Pb)*(tt0/Pb);
+    /* handle higher orbital frequency derivatives if present */
+    if( PulsarCheckParam(params, "FB") ){
+      const REAL8Vector *fb = PulsarGetREAL8VectorParam(params, "FB");
+
+      /* set period from first orbital frequency value */
+      Pb = 1./fb->data[0];
+      orbits = tt0/Pb;
+
+      fac = 1.;
+      for ( UINT4 j=1 ; j < fb->length; j++){
+        fac /= (REAL8)(j+1);
+        orbits += fac*fb->data[j]*pow(tt0,j+1);
+      }
+    }
+    else{
+      orbits = tt0/Pb - 0.5*(pbdot+xpbdot)*(tt0/Pb)*(tt0/Pb);
+    }
+
     norbits = (INT4)orbits;
     if(orbits < 0.0) norbits--;
 
