@@ -21,11 +21,13 @@
 #define _COMPUTEFSTAT_H
 
 #include <lal/LALStdlib.h>
+#include <lal/UserInputParse.h>
 #include <lal/PulsarDataTypes.h>
 #include <lal/LALComputeAM.h>
 #include <lal/LALComputeAM.h>
 #include <lal/SSBtimes.h>
 #include <lal/CWMakeFakeData.h>
+#include <lal/LFTandTSutils.h>
 
 #ifdef  __cplusplus
 extern "C" {
@@ -83,7 +85,7 @@ typedef struct tagFstatInputVector {
 /// Not all options are supported by all \f$\mathcal{F}\f$-statistic methods.
 ///
 typedef enum tagFstatQuantities {
-  FSTATQ_NONE           = 0x00,
+  FSTATQ_NONE           = 0x00,		///< Dont compute F-statistic, still compute buffered quantities
   FSTATQ_2F             = 0x01,         ///< Compute multi-detector \f$2\mathcal{F}\f$.
   FSTATQ_FAFB           = 0x02,         ///< Compute multi-detector \f$F_a\f$ and \f$F_b\f$.
   FSTATQ_2F_PER_DET     = 0x04,         ///< Compute \f$2\mathcal{F}\f$ for each detector.
@@ -133,6 +135,7 @@ typedef struct tagFstatOptionalArgs {
   MultiNoiseFloor *assumeSqrtSX;  	///< Single-sided PSD values to be used for computing SFT noise weights instead of from a running median of the SFTs themselves.
   FstatInput *prevInput;		///< An \c FstatInput structure from a previous call to XLALCreateFstatInput(); may contain common workspace data than can be re-used to save memory.
   BOOLEAN collectTiming;		///< a flag to turn on/off the collection of F-stat-method-specific timing-data
+  BOOLEAN resampFFTPowerOf2;		///< \a Resamp: round up FFT lengths to next power of 2; see #FstatMethodType.
 } FstatOptionalArgs;
 
 ///
@@ -272,11 +275,63 @@ typedef struct tagFstatResults {
 
 } FstatResults;
 
+/// Generic F-stat timing coefficients (times in seconds)
+/// [see https://dcc.ligo.org/LIGO-T1600531-v4 for details]
+/// tauF_eff = tauF_core + b * tauF_buffer
+/// where 'b' denotes the fraction of times per call to XLALComputeFstat() the buffer needed to be recomputed
+/// ==> b = NBufferMisses / NCalls\n"
+///
+/// Note: All timing numbers are averaged over repeated calls to XLALComputeFstat(for fixed-setup)
+///
+typedef struct tagFstatTimingGeneric
+{
+  REAL4 tauF_eff;	//< effective total time per F-stat call to XLALComputeFstat() per detector per output frequency
+  REAL4 tauF_core;	//< core time per output frequency bin per detector, excluding time to compute the buffer
+  REAL4 tauF_buffer;	//< time per detector per frequency bin to re-compute all buffered quantities once
+  UINT4 NFbin;		//< (average over F-stat calls) number of F-stat output frequency bins
+  UINT4 Ndet;		//< number of detectors
+  REAL4 NCalls;		//< number of F-stat calls we average over
+  REAL4 NBufferMisses;	//< number of times the buffer needed to be recomputed
+  char *help;		//< (static) string documenting the generic F-stat timing values
+} FstatTimingGeneric;
+
+#ifdef _COMPUTE_FSTAT_C
+static char FstatTimingGenericHelp[] =
+  "%%%% ----- Generic F-stat timing model (all times in seconds) [see https://dcc.ligo.org/LIGO-T1600531-v4 for details] -----\n"
+  "%%%% tauF_eff:       effective total time per F-stat call to XLALComputeFstat() per detector per output frequency bin\n"
+  "%%%% tauF_core:      core time per output frequency bin per detector, excluding time to compute the buffer\n"
+  "%%%% tauF_buffer:    time per detector per frequency bin to re-compute all buffered quantities once\n"
+  "%%%% NFbin:          (average over F-stat calls) number of F-stat output frequency bins\n"
+  "%%%% Ndet:           number of detectors\n"
+  "%%%%\n"
+  "%%%% => generic F-stat timing:\n"
+  "%%%% tauF_eff = tauF_core + b * tauF_buffer\n"
+  "%%%% where 'b' denotes the fraction of times per call to XLALComputeFstat() the buffer needed to be recomputed\n"
+  "%%%%\n"
+  "%%%% NCalls:         number of F-stat calls we average over\n"
+  "%%%% NBufferMisses:  number of times the buffer needed to be recomputed\n"
+  "%%%% ==> b = NBufferMisses / NCalls\n"
+  "%%%% All timing numbers are averaged over repeated calls to XLALComputeFstat(for fixed-setup).\n"
+  "";
+#endif
+
+#define TIMING_MODEL_MAX_VARS 10
+/// Struct to carry the F-statistic method-specific timing *model* in terms of
+/// a list of variable names and corresponding REAL4 values, including a help-string documenting
+/// the timing model variables.
+/// See https://dcc.ligo.org/LIGO-T1600531-v4 for a more detailed discussion of the F-stat timing model.
+typedef struct tagFstatTimingModel
+{
+  UINT4 numVariables;                           //< number of (method-specific) timing model variables
+  const char* names[TIMING_MODEL_MAX_VARS];     //< array of (static) timing-model variable names
+  REAL4 values[TIMING_MODEL_MAX_VARS];          //< array of timing-model variable values
+  char  *help;                            	//< (static) help string documenting the (method-specific) Fstat timing model
+} FstatTimingModel;
+
 // ---------- API function prototypes ----------
-int XLALFstatMethodIsAvailable ( FstatMethodType i );
-const CHAR *XLALFstatMethodName ( FstatMethodType i );
-const CHAR *XLALFstatMethodHelpString ( void );
-int XLALParseFstatMethodString ( FstatMethodType *Fmethod, const char *s );
+int XLALFstatMethodIsAvailable ( FstatMethodType method );
+const CHAR *XLALFstatMethodName ( FstatMethodType method );
+const UserChoices *XLALFstatMethodChoices ( void );
 
 FstatInputVector* XLALCreateFstatInputVector ( const UINT4 length );
 void XLALDestroyFstatInputVector ( FstatInputVector* input );
@@ -289,13 +344,16 @@ FstatInput *
 XLALCreateFstatInput ( const SFTCatalog *SFTcatalog, const REAL8 minCoverFreq, const REAL8 maxCoverFreq, const REAL8 dFreq,
                        const EphemerisData *ephemerides, const FstatOptionalArgs *optionalArgs );
 
+int XLALGetFstatInputSFTBand ( const FstatInput *input, REAL8 *minFreqFull, REAL8 *maxFreqFull );
 const CHAR *XLALGetFstatInputMethodName ( const FstatInput* input );
 const MultiLALDetector* XLALGetFstatInputDetectors ( const FstatInput* input );
 const MultiLIGOTimeGPSVector* XLALGetFstatInputTimestamps ( const FstatInput* input );
-const MultiNoiseWeights* XLALGetFstatInputNoiseWeights ( const FstatInput* input );
+MultiNoiseWeights* XLALGetFstatInputNoiseWeights ( const FstatInput* input );
 const MultiDetectorStateSeries* XLALGetFstatInputDetectorStates ( const FstatInput* input );
-int XLALGetFstatTiming ( const FstatInput* input, REAL8 *tauF1Buf, REAL8 *tauF1NoBuf );
-int AppendFstatTimingInfo2File ( const FstatInput* input, FILE *fp, BOOLEAN printHeader );
+int  XLALExtractResampledTimeseries ( MultiCOMPLEX8TimeSeries **multiTimeSeries_SRC_a, MultiCOMPLEX8TimeSeries **multiTimeSeries_SRC_b, const FstatInput *input );
+int XLALGetFstatTiming ( const FstatInput* input, FstatTimingGeneric *timingGeneric, FstatTimingModel *timingModel );
+int XLALAppendFstatTiming2File ( const FstatInput* input, FILE *fp, BOOLEAN printHeader );
+int XLALFstatInputTimeslice ( FstatInput** slice, const FstatInput* input, const LIGOTimeGPS *minStartGPS, const LIGOTimeGPS *maxStartGPS);
 
 #ifdef SWIG // SWIG interface directives
 SWIGLAL(INOUT_STRUCTS(FstatResults**, Fstats));
@@ -307,6 +365,8 @@ void XLALDestroyFstatInput ( FstatInput* input );
 void XLALDestroyFstatResults ( FstatResults* Fstats );
 int XLALAdd4ToFstatResults ( FstatResults* Fstats );
 
+REAL4 XLALComputeFstatFromFaFb ( COMPLEX8 Fa, COMPLEX8 Fb, REAL4 A, REAL4 B, REAL4 C, REAL4 E, REAL4 Dinv );
+
 REAL4 XLALComputeFstatFromAtoms ( const MultiFstatAtomVector *multiFstatAtoms, const INT4 X );
 
 // @}
