@@ -38,6 +38,13 @@
 
 /*---------- Global variables ----------*/
 
+const UserChoices SSBprecisionChoices = {
+  { SSBPREC_NEWTONIAN,		"newtonian" },
+  { SSBPREC_RELATIVISTIC,	"relativistic" },
+  { SSBPREC_RELATIVISTICOPT,  	"relativisticopt" },
+  { SSBPREC_DMOFF,              "DMoff"},
+};
+
 /*---------- internal prototypes ----------*/
 
 static double gsl_E_solver ( double E, void *p );
@@ -320,7 +327,7 @@ XLALAddBinaryTimes ( SSBtimes **tSSBOut,			//!< [out] reference-time offsets in
       REAL8 x0 = fracOrb_i * LAL_TWOPI;
       REAL8 E_i;              // eccentric anomaly at emission of the wavefront arriving in SSB at tSSB
       { // ---------- use GSL for the root-finding
-        const gsl_root_fsolver_type *T = gsl_root_fsolver_bisection;
+        const gsl_root_fsolver_type *T = gsl_root_fsolver_brent;
         gsl_root_fsolver *s = gsl_root_fsolver_alloc(T);
         REAL8 E_lo = 0, E_hi = LAL_TWOPI;	// gauge-choice mod (2pi)
         gsl_function F;
@@ -631,6 +638,16 @@ XLALGetSSBtimes ( const DetectorStateSeries *DetectorStates,	/**< [in] detector-
 
       break;
 
+    case SSBPREC_DMOFF:	/* switch off all demodulation terms */
+
+      for ( UINT4 i = 0; i < numSteps; i++ )
+        {
+          DetectorState *state = &(DetectorStates->data[i]);
+          ret->DeltaT->data[i] = XLALGPSGetREAL8 ( &state->tGPS ) - refTimeREAL8;
+          ret->Tdot->data[i]   = 1.0;
+        } /* for i < numSteps */
+      break;
+
     default:
       XLAL_ERROR_NULL (XLAL_EFAILED, "\n?? Something went wrong.. this should never be called!\n\n" );
       break;
