@@ -35,6 +35,7 @@ extern "C" {
 
 /*---------- exported INCLUDES ----------*/
 #include <lal/LALStdlib.h>
+#include <lal/UserInputParse.h>
 #include <lal/PulsarDataTypes.h>
 #include <lal/DetectorStates.h>
 
@@ -45,9 +46,13 @@ typedef enum tagSSBprecision {
   SSBPREC_NEWTONIAN,		/**< simple Newtonian: \f$\tau = t + \vec{r}\cdot\vec{n}/c\f$ */
   SSBPREC_RELATIVISTIC,		/**< detailed relativistic: \f$\tau=\tau(t; \vec{n}, \vec{r})\f$ */
   SSBPREC_RELATIVISTICOPT,  	/**< optimized relativistic, numerically equivalent to #SSBPREC_RELATIVISTIC, but faster */
+  SSBPREC_DMOFF,                /**< switch off all demodulatoin terms */
   SSBPREC_LAST			/**< end marker */
 } SSBprecision;
 
+/** Static array of all #SSBprecision choices, for use by the UserInput module parsing routines */
+extern const UserChoices SSBprecisionChoices;
+
 /** Simple container for two REAL8-vectors, namely the SSB-timings DeltaT_alpha  and Tdot_alpha,
  * with one entry per SFT-timestamp. These are required input for XLALNewDemod().
  * We also store the SSB reference-time tau0.
