@@ -26,7 +26,7 @@
 
 #include <lal/SynthesizeCWDraws.h>
 
-#include <lal/EstimateAmplitudeParams.h>
+#include <lal/FstatisticTools.h>
 
 /*---------- DEFINES ----------*/
 #define SQ(x) ((x)*(x))
