@@ -21,6 +21,7 @@
 #include <math.h>
 #include <sys/times.h>
 
+#include <lal/LALPulsarVCSInfo.h>
 #include <lal/ComputeFstat.h>
 #include <lal/LALgetopt.h>
 #include <lal/LALInitBarycenter.h>
@@ -90,7 +91,7 @@ main ( int argc, char *argv[] )
 
   /* read cmdline & cfgfile  */
   BOOLEAN should_exit = 0;
-  XLAL_CHECK( XLALUserVarReadAllInput( &should_exit, argc, argv ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK( XLALUserVarReadAllInput( &should_exit, argc, argv, lalPulsarVCSInfoList ) == XLAL_SUCCESS, XLAL_EFUNC );
   if ( should_exit ) {
     exit (1);
   }
@@ -100,8 +101,8 @@ main ( int argc, char *argv[] )
   REAL8 startTimeREAL8 	= 714180733;
   REAL8 duration 	= 180000;	/* 50 hours */
   REAL8 Tsft 		= 1800;		/* assume 30min SFTs */
-  char earthEphem[] 	= TEST_DATA_DIR "earth00-19-DE200.dat.gz";
-  char sunEphem[]   	= TEST_DATA_DIR "sun00-19-DE200.dat.gz";
+  char earthEphem[] 	= TEST_PKG_DATA_DIR "earth00-19-DE200.dat.gz";
+  char sunEphem[]   	= TEST_PKG_DATA_DIR "sun00-19-DE200.dat.gz";
 
   //REAL8 tolerance = 2e-10;	/* same algorithm, should be basically identical results */
 
