@@ -188,8 +188,8 @@ void RunGeneratePulsarSignalTest(LALStatus *status)
   CHAR IFO[6] = "LHO";
   EphemerisData *edat = NULL;
 
-  char earthFile[] = TEST_DATA_DIR "earth00-19-DE405.dat.gz";
-  char sunFile[]   = TEST_DATA_DIR "sun00-19-DE405.dat.gz";
+  char earthFile[] = TEST_PKG_DATA_DIR "earth00-19-DE405.dat.gz";
+  char sunFile[]   = TEST_PKG_DATA_DIR "sun00-19-DE405.dat.gz";
 
   /* containers for sky position and spindown data */
   REAL8 **skyPosData;
@@ -328,7 +328,7 @@ void RunGeneratePulsarSignalTest(LALStatus *status)
   XLAL_CHECK_LAL (status, ( edat = XLALInitBarycenter( earthFile, sunFile ) ) != NULL, XLAL_EFUNC);
 
   /* Allocate memory for PulsarSignalParams and initialize */
-  pPulsarSignalParams = (PulsarSignalParams *)LALMalloc(sizeof(PulsarSignalParams));
+  pPulsarSignalParams = (PulsarSignalParams *)LALCalloc(1, sizeof(PulsarSignalParams));
   pPulsarSignalParams->pulsar.position.system = COORDINATESYSTEM_EQUATORIAL;
   pPulsarSignalParams->pulsar.spindown = NULL;
   if (numSpinDown > 0) {
