@@ -23,7 +23,7 @@
  * \date 2001
  * \file
  * \ingroup LALBarycenter_h
- * \brief  Tests the routine LALBarycenter(): exercises some of the error
+ * \brief  Tests the routine XLALBarycenter(): exercises some of the error
  * conditions and makes sure that they work.
  *
  * ### Program <tt>LALBarycenterTest.c</tt> ###
@@ -37,7 +37,7 @@
  *
  * ### Description ###
  *
- * This program demonstrates the use of LALBarycenter.c.
+ * This program demonstrates the use of XLALBarycenter.c.
  * The two ephemeris files specified in the EphemerisFilenames
  * structure (e.g., for data taken in 1998, <tt>sun98.dat</tt> and <tt>earth98.dat</tt>)
  * are assumed to be in the same directory as the program as
@@ -51,23 +51,6 @@
 #include <lal/Date.h>
 #include <lal/LogPrintf.h>
 
-/** \name Error codes */
-/*@{*/
-#define LALBARYCENTERTESTC_ENOM 0
-#define LALBARYCENTERTESTC_EOPEN 1
-#define LALBARYCENTERTESTC_EOUTOFRANGEE  4
-#define LALBARYCENTERTESTC_EOUTOFRANGES  8
-#define LALBARYCENTERTESTC_EBADSOURCEPOS 16
-#define LALBARYCENTERTESTC_EEPHFILE 32
-
-#define LALBARYCENTERTESTC_MSGENOM "Nominal exit"
-#define LALBARYCENTERTESTC_MSGEOPEN "Error checking failed to catch missing ephemeris file."
-#define LALBARYCENTERTESTC_MSGEOUTOFRANGEE "Failed to catch that tgps not in range of earth.dat file"
-#define LALBARYCENTERTESTC_MSGEOUTOFRANGES "Failed to catch that tgps not in range of sun.dat file"
-#define LALBARYCENTERTESTC_MSGEBADSOURCEPOS "Failed to catch bad source position"
-#define LALBARYCENTERTESTC_MSGEEPHFILE "Failed to catch error reading ephemeris file."
-/*@}*/
-
 /** \cond DONT_DOXYGEN */
 
 /* ----- internal prototype ---------- */
@@ -96,67 +79,36 @@ const INT4 t1998 = 630720013-730*86400-1;	/* gps at Jan 1,1998 00:00:00 UTC*/
 int
 main( void )
 {
-  static LALStatus status;
 
   char eEphFileBad[] = TEST_DATA_DIR "earth47.dat";
   char eEphFile[] = TEST_DATA_DIR "earth98.dat";
   char sEphFile[] = TEST_DATA_DIR "sun98.dat";
 
   /* Checking response if data files not present */
-  EphemerisData edat;
-  edat.ephiles.earthEphemeris = eEphFileBad;
-  edat.ephiles.sunEphemeris   = sEphFile;
-  LALInitBarycenter(&status, &edat);
-  if ( status.statusCode != LALINITBARYCENTERH_EOPEN)
-    {
-      XLALPrintError( "Got error code %d and message '%s', but expected error code %d\n", status.statusCode, status.statusDescription, LALINITBARYCENTERH_EOPEN);
-      return LALBARYCENTERTESTC_EOPEN;
-    }
-  else
-    {
-      // XLALPrintError ("==================== this error is as expected and OK!! ==================== \n");
-      xlalErrno = 0;
-    }
+  EphemerisData *edat = XLALInitBarycenter( eEphFileBad, sEphFile );
+  XLAL_CHECK_MAIN( edat == NULL, XLAL_EFAILED, "Expected XLALInitBarycenter( '%s', '%s' ) to fail!", eEphFileBad, sEphFile );
+  XLALClearErrno();
 
   /* Now inputting kosher ephemeris. files and leap sec, to illustrate
    * proper usage. The real, serious TEST of the code is a script written
-   * by Rejean Dupuis comparing LALBarycenter to TEMPO for thousands
+   * by Rejean Dupuis comparing XLALBarycenter to TEMPO for thousands
    * of source positions and times.
    */
-  edat.ephiles.earthEphemeris = eEphFile;
-  edat.ephiles.sunEphemeris = sEphFile;
-  LALInitBarycenter(&status, &edat);
-  if ( status.statusCode ) {
-    XLALPrintError ("LALInitBarycenter() failed with code %d\n", status.statusCode);
-    return XLAL_EFAILED;
-  }
-
-  /* ===== now test equivalence of new XLALInitBarycenter() function ========== */
-  EphemerisData *edat_xlal;
-  if ( ( edat_xlal = XLALInitBarycenter ( eEphFile, sEphFile )) == NULL ) {
-    XLALPrintError ("Something failed in XLALInitBarycenter(), errno =%d\n", xlalErrno );
-    return XLAL_EFAILED;
-  }
-  if ( compare_ephemeris ( &edat, edat_xlal ) != XLAL_SUCCESS ) {
-    XLALPrintError ("Equivalence test failed between XLALInitEphemeris() and LALInitEphemeris()\n" );
-    return XLAL_EFAILED;
-  }
-  XLALDestroyEphemerisData ( edat_xlal );
+  XLAL_CHECK_MAIN( ( edat = XLALInitBarycenter( eEphFile, sEphFile ) ) != NULL, XLAL_EFUNC );
 
   /* ========================================================================== */
 
 
- /* The routines using LALBarycenter package, the code above, leading
-    up LALInitBarycenter call, should be near top of main. The idea is
+ /* The routines using XLALBarycenter package, the code above, leading
+    up XLALInitBarycenter() call, should be near top of main. The idea is
     that ephemeris data is read into RAM once, at the beginning.
 
     NOTE that the only part of the piece of the LALDetector structure
     baryinput.site that has to be filled in by the driver code is
     the 3-vector: baryinput.site.location[] .
 
-    NOTE that the driver code that calls LALInitBarycenter must
-    LALFree(edat->ephemE) and LALFree(edat->ephemS).
-    The driver code that calls LALBarycenter must LALFree(edat).
+    NOTE that the driver code that calls XLALInitBarycenter() must
+    XLALDestroyEphemerisData(edat).
  */
 
   /* Now getting coords for detector */
@@ -169,44 +121,30 @@ main( void )
   baryinput.site.location[2]=cachedDetector.location[2]/LAL_C_SI;
 
   EarthState earth;
-  EarthState earth_xlal;
-  EmissionTime  emit, emit_xlal, emit_opt;
+  EmissionTime emit, emit_opt;
 
   /* ----- Checking error messages when the timestamp is not within the 1-yr ephemeris files */
   LIGOTimeGPS tGPS = {t1998+5e7, 0 };
-  LALBarycenterEarth ( &status, &earth, &tGPS, &edat );
-  if ( status.statusCode == 0 ) {
-    XLALPrintError ( "LALBarycenterEarth() succeeded but expected to get error\n");
-    return LALBARYCENTERTESTC_EOUTOFRANGEE;
-  } else {
-    XLALPrintError ("==================== this error is as expected and OK!! ==================== \n");
-    xlalErrno = 0;
-  }
+  XLAL_CHECK_MAIN( XLALBarycenterEarth ( &earth, &tGPS, edat ) == XLAL_FAILURE, XLAL_EFAILED, "Expected XLALBarycenterEarth() to fail!" );
+  XLALClearErrno();
 
   /* next try calling for bad choice of RA,DEC (e.g., something sensible in degrees, but radians)*/
   tGPS.gpsSeconds = t1998+3600;
-  LALBarycenterEarth ( &status, &earth, &tGPS, &edat );
+  XLAL_CHECK_MAIN( XLALBarycenterEarth ( &earth, &tGPS, edat ) == XLAL_SUCCESS, XLAL_EFUNC );
 
   baryinput.alpha= 120;
   baryinput.delta = 60;
   baryinput.dInv = 0;
 
-  LALBarycenter ( &status, &emit, &baryinput, &earth );
-  if ( status.statusCode == 0 ) {
-    XLALPrintError( "LALBarycenter() succeeded but expected to get error\n" );
-    return LALBARYCENTERTESTC_EBADSOURCEPOS;
-  } else {
-    XLALPrintError ("==================== this error is as expected and OK!! ==================== \n");
-    xlalErrno = 0;
-  }
+  XLAL_CHECK_MAIN( XLALBarycenter ( &emit, &baryinput, &earth ) == XLAL_FAILURE, XLAL_EFAILED, "Expected XLALBarycenter() to fail!" );
+  XLALClearErrno();
 
   /* ---------- Now running program w/o errors, to illustrate proper use. ---------- */
-  EmissionTime XLAL_INIT_DECL(maxDiff);
   EmissionTime XLAL_INIT_DECL(maxDiffOpt);
   REAL8 tic, toc;
   UINT4 NRepeat = 1;
   UINT4 counter = 0;
-  REAL8 tau_lal = 0, tau_xlal = 0, tau_opt = 0;
+  REAL8 tau = 0, tau_opt = 0;
   BarycenterBuffer *buffer = NULL;
 
   unsigned int seed = XLALGetTimeOfDay();
@@ -226,44 +164,22 @@ main( void )
           XLALGPSSetREAL8( &tGPS, tPulse );
           baryinput.tgps = tGPS;
 
-          /* ----- old LAL interface ---------- */
-          LALBarycenterEarth ( &status, &earth, &tGPS, &edat);
-          if ( status.statusCode ) {
-            XLALPrintError ("LALBarycenterEarth() failed with code %d\n", status.statusCode);
-            return XLAL_EFAILED;
-          }
-
-          tic = XLALGetTimeOfDay();
-          for ( UINT4 l = 0; l < NRepeat; l++ )
-            LALBarycenter ( &status, &emit, &baryinput, &earth );
-          toc = XLALGetTimeOfDay();
-          tau_lal += ( toc - tic ) / NRepeat;
-          if ( status.statusCode ) {
-            XLALPrintError ("LALBarycenter() failed with code %d\n", status.statusCode);
-            return XLAL_EFAILED;
-          }
-
-          /* ----- new XLAL interface ---------- */
-          XLAL_CHECK ( XLALBarycenterEarth ( &earth_xlal, &tGPS, &edat ) == XLAL_SUCCESS, XLAL_EFAILED );
+          XLAL_CHECK ( XLALBarycenterEarth ( &earth, &tGPS, edat ) == XLAL_SUCCESS, XLAL_EFAILED );
           tic = XLALGetTimeOfDay();
           for ( UINT4 l = 0; l < NRepeat; l ++ )
-            XLAL_CHECK ( XLALBarycenter ( &emit_xlal, &baryinput, &earth_xlal ) == XLAL_SUCCESS, XLAL_EFAILED );
+            XLAL_CHECK ( XLALBarycenter ( &emit, &baryinput, &earth ) == XLAL_SUCCESS, XLAL_EFAILED );
           toc = XLALGetTimeOfDay();
-          tau_xlal += ( toc - tic ) / NRepeat;
-
-          /* collect maximal deviations over all struct-fields of 'emit' */
-          EmissionTime thisDiff;
-          diffEmissionTime ( &thisDiff, &emit, &emit_xlal );
-          absmaxEmissionTime ( &maxDiff, &maxDiff, &thisDiff );
+          tau += ( toc - tic ) / NRepeat;
 
           /* ----- optimized XLAL version with buffering ---------- */
           tic = XLALGetTimeOfDay();
           for ( UINT4 l = 0; l < NRepeat; l ++ )
-            XLAL_CHECK ( XLALBarycenterOpt ( &emit_opt, &baryinput, &earth_xlal, &buffer ) == XLAL_SUCCESS, XLAL_EFAILED );
+            XLAL_CHECK ( XLALBarycenterOpt ( &emit_opt, &baryinput, &earth, &buffer ) == XLAL_SUCCESS, XLAL_EFAILED );
           toc = XLALGetTimeOfDay();
           tau_opt += ( toc - tic ) / NRepeat;
 
           /* collect maximal deviations over all struct-fields of 'emit' */
+          EmissionTime thisDiff;
           diffEmissionTime ( &thisDiff, &emit, &emit_opt );
           absmaxEmissionTime ( &maxDiffOpt, &maxDiffOpt, &thisDiff );
 
@@ -277,15 +193,10 @@ main( void )
 
   /* ----- check differences in results ---------- */
   REAL8 tolerance = 1e-9;	// in seconds: can't go beyond nanosecond precision due to GPS limitation
-  REAL8 maxEmitDiff = maxErrInEmissionTime ( &maxDiff );
   REAL8 maxEmitDiffOpt = maxErrInEmissionTime ( &maxDiffOpt );
-  XLALPrintInfo ( "Max error (in seconds) between LALBarycenter() and XLALBarycenter()     = %g s (tolerance = %g s)\n", maxEmitDiff, tolerance );
-  XLAL_CHECK ( maxEmitDiff < tolerance, XLAL_EFAILED,
-               "Max error (in seconds) between LALBarycenter() and XLALBarycenter()  = %g s, exceeding tolerance of %g s\n", maxEmitDiff, tolerance );
-
-  XLALPrintInfo ( "Max error (in seconds) between LALBarycenter() and XLALBarycenterOpt()  = %g s (tolerance = %g s)\n", maxEmitDiffOpt, tolerance );
+  XLALPrintInfo ( "Max error (in seconds) between XLALBarycenter() and XLALBarycenterOpt()  = %g s (tolerance = %g s)\n", maxEmitDiffOpt, tolerance );
   XLAL_CHECK ( maxEmitDiffOpt < tolerance, XLAL_EFAILED,
-               "Max error (in seconds) between LALBarycenter() and XLALBarycenterOpt()  = %g s, exceeding tolerance of %g s\n",
+               "Max error (in seconds) between XLALBarycenter() and XLALBarycenterOpt()  = %g s, exceeding tolerance of %g s\n",
                maxEmitDiffOpt, tolerance );
   printf ( "%g	%g %d %d %g	%g %g %g	%g %g %g\n",
            maxEmitDiffOpt,
@@ -296,93 +207,91 @@ main( void )
 
   /* ----- output runtimes ---------- */
   XLALPrintError ("Runtimes per function-call, averaged over %g calls\n", 1.0 * NRepeat * counter );
-  XLALPrintError ("LALBarycenter() 	%g s\n", tau_lal / counter );
-  XLALPrintError ("XLALBarycenter()	%g s (= %.1f %%)\n", tau_xlal / counter, - 100 * (tau_lal - tau_xlal ) / tau_lal );
-  XLALPrintError ("XLALBarycenterOpt()	%g s (= %.1f %%)\n", tau_opt / counter,  - 100 * (tau_lal - tau_opt ) / tau_lal );
+  XLALPrintError ("XLALBarycenter() 	%g s\n", tau / counter );
+  XLALPrintError ("XLALBarycenterOpt()	%g s (= %.1f %%)\n", tau_opt / counter,  - 100 * (tau - tau_opt ) / tau );
 
   /* ===== test XLALRestrictEphemerisData() ===== */
   XLALPrintInfo("\n\nTesting XLALRestrictEphemerisData() ... ");
   {
-    XLAL_CHECK( edat.nentriesS >= 100, XLAL_EFAILED );
-    const INT4 orig_nentriesS = edat.nentriesS;
+    XLAL_CHECK( edat->nentriesS >= 100, XLAL_EFAILED );
+    const INT4 orig_nentriesS = edat->nentriesS;
     for (INT4 i = 1; i <= 4; ++i) {
       REAL8 start, end;
       LIGOTimeGPS startGPS, endGPS;
       INT4 diff_nentriesS;
 
-      start = edat.ephemS[2*i].gps;
-      end = edat.ephemS[edat.nentriesS - 1 - 3*i].gps;
+      start = edat->ephemS[2*i].gps;
+      end = edat->ephemS[edat->nentriesS - 1 - 3*i].gps;
       XLAL_CHECK( XLALGPSSetREAL8(&startGPS, start) != NULL, XLAL_EFUNC );
       XLAL_CHECK( XLALGPSSetREAL8(&endGPS, end) != NULL, XLAL_EFUNC );
-      XLAL_CHECK( XLALRestrictEphemerisData(&edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
-      XLAL_CHECK( edat.ephemS[0].gps == start, XLAL_EFAILED, "\nTest S%dA FAILED: %0.9f != start %0.9f\n", i, edat.ephemS[0].gps, start );
-      XLAL_CHECK( edat.ephemS[edat.nentriesS - 1].gps == end, XLAL_EFAILED, "\nTest S%dA FAILED: end %0.9f != %0.9f\n", i, edat.ephemS[edat.nentriesS - 1].gps, end );
+      XLAL_CHECK( XLALRestrictEphemerisData(edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
+      XLAL_CHECK( edat->ephemS[0].gps == start, XLAL_EFAILED, "\nTest S%dA FAILED: %0.9f != start %0.9f\n", i, edat->ephemS[0].gps, start );
+      XLAL_CHECK( edat->ephemS[edat->nentriesS - 1].gps == end, XLAL_EFAILED, "\nTest S%dA FAILED: end %0.9f != %0.9f\n", i, edat->ephemS[edat->nentriesS - 1].gps, end );
       diff_nentriesS = ((i*i + i) * 5) / 2 + 2*(i-1);
-      XLAL_CHECK( orig_nentriesS - edat.nentriesS == diff_nentriesS, XLAL_EFAILED, "\nTest S%dA FAILED: nentries %d != %d\n", i, orig_nentriesS - edat.nentriesS, diff_nentriesS );
+      XLAL_CHECK( orig_nentriesS - edat->nentriesS == diff_nentriesS, XLAL_EFAILED, "\nTest S%dA FAILED: nentries %d != %d\n", i, orig_nentriesS - edat->nentriesS, diff_nentriesS );
 
-      XLAL_CHECK( XLALGPSSetREAL8(&startGPS, start + 0.5*edat.dtStable) != NULL, XLAL_EFUNC );
-      XLAL_CHECK( XLALGPSSetREAL8(&endGPS, end - 0.5*edat.dtStable) != NULL, XLAL_EFUNC );
-      start = edat.ephemS[0].gps;
-      end = edat.ephemS[edat.nentriesS - 1].gps;
-      XLAL_CHECK( XLALRestrictEphemerisData(&edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
-      XLAL_CHECK( edat.ephemS[0].gps == start, XLAL_EFAILED, "\nTest S%dB FAILED: start %0.9f != %0.9f\n", i, edat.ephemS[0].gps, start );
-      XLAL_CHECK( edat.ephemS[edat.nentriesS - 1].gps == end, XLAL_EFAILED, "\nTest S%dB FAILED: end %0.9f != %0.9f\n", i, edat.ephemS[edat.nentriesS - 1].gps, end );
+      XLAL_CHECK( XLALGPSSetREAL8(&startGPS, start + 0.5*edat->dtStable) != NULL, XLAL_EFUNC );
+      XLAL_CHECK( XLALGPSSetREAL8(&endGPS, end - 0.5*edat->dtStable) != NULL, XLAL_EFUNC );
+      start = edat->ephemS[0].gps;
+      end = edat->ephemS[edat->nentriesS - 1].gps;
+      XLAL_CHECK( XLALRestrictEphemerisData(edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
+      XLAL_CHECK( edat->ephemS[0].gps == start, XLAL_EFAILED, "\nTest S%dB FAILED: start %0.9f != %0.9f\n", i, edat->ephemS[0].gps, start );
+      XLAL_CHECK( edat->ephemS[edat->nentriesS - 1].gps == end, XLAL_EFAILED, "\nTest S%dB FAILED: end %0.9f != %0.9f\n", i, edat->ephemS[edat->nentriesS - 1].gps, end );
       diff_nentriesS = ((i*i + i) * 5) / 2 + 2*(i-1);
-      XLAL_CHECK( orig_nentriesS - edat.nentriesS == diff_nentriesS, XLAL_EFAILED, "\nTest S%dB FAILED: nentries %d != %d\n", i, orig_nentriesS - edat.nentriesS, diff_nentriesS );
+      XLAL_CHECK( orig_nentriesS - edat->nentriesS == diff_nentriesS, XLAL_EFAILED, "\nTest S%dB FAILED: nentries %d != %d\n", i, orig_nentriesS - edat->nentriesS, diff_nentriesS );
 
-      XLAL_CHECK( XLALGPSSetREAL8(&startGPS, start + 1.5*edat.dtStable) != NULL, XLAL_EFUNC );
-      XLAL_CHECK( XLALGPSSetREAL8(&endGPS, end - 1.5*edat.dtStable) != NULL, XLAL_EFUNC );
-      start = edat.ephemS[1].gps;
-      end = edat.ephemS[edat.nentriesS - 2].gps;
-      XLAL_CHECK( XLALRestrictEphemerisData(&edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
-      XLAL_CHECK( edat.ephemS[0].gps == start, XLAL_EFAILED, "\nTest S%dC FAILED: start %0.9f != %0.9f\n", i, edat.ephemS[0].gps, start );
-      XLAL_CHECK( edat.ephemS[edat.nentriesS - 1].gps == end, XLAL_EFAILED, "\nTest S%dC FAILED: end %0.9f != %0.9f\n", i, edat.ephemS[edat.nentriesS - 1].gps, end );
+      XLAL_CHECK( XLALGPSSetREAL8(&startGPS, start + 1.5*edat->dtStable) != NULL, XLAL_EFUNC );
+      XLAL_CHECK( XLALGPSSetREAL8(&endGPS, end - 1.5*edat->dtStable) != NULL, XLAL_EFUNC );
+      start = edat->ephemS[1].gps;
+      end = edat->ephemS[edat->nentriesS - 2].gps;
+      XLAL_CHECK( XLALRestrictEphemerisData(edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
+      XLAL_CHECK( edat->ephemS[0].gps == start, XLAL_EFAILED, "\nTest S%dC FAILED: start %0.9f != %0.9f\n", i, edat->ephemS[0].gps, start );
+      XLAL_CHECK( edat->ephemS[edat->nentriesS - 1].gps == end, XLAL_EFAILED, "\nTest S%dC FAILED: end %0.9f != %0.9f\n", i, edat->ephemS[edat->nentriesS - 1].gps, end );
       diff_nentriesS = ((i*i + i) * 5) / 2 + 2*i;
-      XLAL_CHECK( orig_nentriesS - edat.nentriesS == diff_nentriesS, XLAL_EFAILED, "\nTest S%dC FAILED: nentries %d != %d\n", i, orig_nentriesS - edat.nentriesS, diff_nentriesS );
+      XLAL_CHECK( orig_nentriesS - edat->nentriesS == diff_nentriesS, XLAL_EFAILED, "\nTest S%dC FAILED: nentries %d != %d\n", i, orig_nentriesS - edat->nentriesS, diff_nentriesS );
     }
 
-    XLAL_CHECK( edat.nentriesE >= 100, XLAL_EFAILED );
-    const INT4 orig_nentriesE = edat.nentriesE;
+    XLAL_CHECK( edat->nentriesE >= 100, XLAL_EFAILED );
+    const INT4 orig_nentriesE = edat->nentriesE;
     for (INT4 i = 1; i <= 4; ++i) {
       REAL8 start, end;
       LIGOTimeGPS startGPS, endGPS;
       INT4 diff_nentriesE;
 
-      start = edat.ephemE[2*i].gps;
-      end = edat.ephemE[edat.nentriesE - 1 - 3*i].gps;
+      start = edat->ephemE[2*i].gps;
+      end = edat->ephemE[edat->nentriesE - 1 - 3*i].gps;
       XLAL_CHECK( XLALGPSSetREAL8(&startGPS, start) != NULL, XLAL_EFUNC );
       XLAL_CHECK( XLALGPSSetREAL8(&endGPS, end) != NULL, XLAL_EFUNC );
-      XLAL_CHECK( XLALRestrictEphemerisData(&edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
-      XLAL_CHECK( edat.ephemE[0].gps == start, XLAL_EFAILED, "\nTest E%dA FAILED: %0.9f != start %0.9f\n", i, edat.ephemE[0].gps, start );
-      XLAL_CHECK( edat.ephemE[edat.nentriesE - 1].gps == end, XLAL_EFAILED, "\nTest E%dA FAILED: end %0.9f != %0.9f\n", i, edat.ephemE[edat.nentriesE - 1].gps, end );
+      XLAL_CHECK( XLALRestrictEphemerisData(edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
+      XLAL_CHECK( edat->ephemE[0].gps == start, XLAL_EFAILED, "\nTest E%dA FAILED: %0.9f != start %0.9f\n", i, edat->ephemE[0].gps, start );
+      XLAL_CHECK( edat->ephemE[edat->nentriesE - 1].gps == end, XLAL_EFAILED, "\nTest E%dA FAILED: end %0.9f != %0.9f\n", i, edat->ephemE[edat->nentriesE - 1].gps, end );
       diff_nentriesE = ((i*i + i) * 5) / 2 + 2*(i-1);
-      XLAL_CHECK( orig_nentriesE - edat.nentriesE == diff_nentriesE, XLAL_EFAILED, "\nTest E%dA FAILED: nentries %d != %d\n", i, orig_nentriesE - edat.nentriesE, diff_nentriesE );
+      XLAL_CHECK( orig_nentriesE - edat->nentriesE == diff_nentriesE, XLAL_EFAILED, "\nTest E%dA FAILED: nentries %d != %d\n", i, orig_nentriesE - edat->nentriesE, diff_nentriesE );
 
-      XLAL_CHECK( XLALGPSSetREAL8(&startGPS, start + 0.5*edat.dtEtable) != NULL, XLAL_EFUNC );
-      XLAL_CHECK( XLALGPSSetREAL8(&endGPS, end - 0.5*edat.dtEtable) != NULL, XLAL_EFUNC );
-      start = edat.ephemE[0].gps;
-      end = edat.ephemE[edat.nentriesE - 1].gps;
-      XLAL_CHECK( XLALRestrictEphemerisData(&edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
-      XLAL_CHECK( edat.ephemE[0].gps == start, XLAL_EFAILED, "\nTest E%dB FAILED: start %0.9f != %0.9f\n", i, edat.ephemE[0].gps, start );
-      XLAL_CHECK( edat.ephemE[edat.nentriesE - 1].gps == end, XLAL_EFAILED, "\nTest E%dB FAILED: end %0.9f != %0.9f\n", i, edat.ephemE[edat.nentriesE - 1].gps, end );
+      XLAL_CHECK( XLALGPSSetREAL8(&startGPS, start + 0.5*edat->dtEtable) != NULL, XLAL_EFUNC );
+      XLAL_CHECK( XLALGPSSetREAL8(&endGPS, end - 0.5*edat->dtEtable) != NULL, XLAL_EFUNC );
+      start = edat->ephemE[0].gps;
+      end = edat->ephemE[edat->nentriesE - 1].gps;
+      XLAL_CHECK( XLALRestrictEphemerisData(edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
+      XLAL_CHECK( edat->ephemE[0].gps == start, XLAL_EFAILED, "\nTest E%dB FAILED: start %0.9f != %0.9f\n", i, edat->ephemE[0].gps, start );
+      XLAL_CHECK( edat->ephemE[edat->nentriesE - 1].gps == end, XLAL_EFAILED, "\nTest E%dB FAILED: end %0.9f != %0.9f\n", i, edat->ephemE[edat->nentriesE - 1].gps, end );
       diff_nentriesE = ((i*i + i) * 5) / 2 + 2*(i-1);
-      XLAL_CHECK( orig_nentriesE - edat.nentriesE == diff_nentriesE, XLAL_EFAILED, "\nTest E%dB FAILED: nentries %d != %d\n", i, orig_nentriesE - edat.nentriesE, diff_nentriesE );
+      XLAL_CHECK( orig_nentriesE - edat->nentriesE == diff_nentriesE, XLAL_EFAILED, "\nTest E%dB FAILED: nentries %d != %d\n", i, orig_nentriesE - edat->nentriesE, diff_nentriesE );
 
-      XLAL_CHECK( XLALGPSSetREAL8(&startGPS, start + 1.5*edat.dtEtable) != NULL, XLAL_EFUNC );
-      XLAL_CHECK( XLALGPSSetREAL8(&endGPS, end - 1.5*edat.dtEtable) != NULL, XLAL_EFUNC );
-      start = edat.ephemE[1].gps;
-      end = edat.ephemE[edat.nentriesE - 2].gps;
-      XLAL_CHECK( XLALRestrictEphemerisData(&edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
-      XLAL_CHECK( edat.ephemE[0].gps == start, XLAL_EFAILED, "\nTest E%dC FAILED: start %0.9f != %0.9f\n", i, edat.ephemE[0].gps, start );
-      XLAL_CHECK( edat.ephemE[edat.nentriesE - 1].gps == end, XLAL_EFAILED, "\nTest E%dC FAILED: end %0.9f != %0.9f\n", i, edat.ephemE[edat.nentriesE - 1].gps, end );
+      XLAL_CHECK( XLALGPSSetREAL8(&startGPS, start + 1.5*edat->dtEtable) != NULL, XLAL_EFUNC );
+      XLAL_CHECK( XLALGPSSetREAL8(&endGPS, end - 1.5*edat->dtEtable) != NULL, XLAL_EFUNC );
+      start = edat->ephemE[1].gps;
+      end = edat->ephemE[edat->nentriesE - 2].gps;
+      XLAL_CHECK( XLALRestrictEphemerisData(edat, &startGPS, &endGPS) == XLAL_SUCCESS, XLAL_EFUNC );
+      XLAL_CHECK( edat->ephemE[0].gps == start, XLAL_EFAILED, "\nTest E%dC FAILED: start %0.9f != %0.9f\n", i, edat->ephemE[0].gps, start );
+      XLAL_CHECK( edat->ephemE[edat->nentriesE - 1].gps == end, XLAL_EFAILED, "\nTest E%dC FAILED: end %0.9f != %0.9f\n", i, edat->ephemE[edat->nentriesE - 1].gps, end );
       diff_nentriesE = ((i*i + i) * 5) / 2 + 2*i;
-      XLAL_CHECK( orig_nentriesE - edat.nentriesE == diff_nentriesE, XLAL_EFAILED, "\nTest E%dC FAILED: nentries %d != %d\n", i, orig_nentriesE - edat.nentriesE, diff_nentriesE );
+      XLAL_CHECK( orig_nentriesE - edat->nentriesE == diff_nentriesE, XLAL_EFAILED, "\nTest E%dC FAILED: nentries %d != %d\n", i, orig_nentriesE - edat->nentriesE, diff_nentriesE );
     }
   }
   XLALPrintInfo("PASSED\n\n");
 
-  LALFree(edat.ephemE);
-  LALFree(edat.ephemS);
+  XLALDestroyEphemerisData(edat);
 
   LALCheckMemoryLeaks();
 
