@@ -102,7 +102,7 @@ static int SerialisationTest(
   // Count number of points
   const UINT8 total = XLALTotalLatticeTilingPoints( itr );
   XLAL_CHECK( total > 0, XLAL_EFUNC );
-  XLAL_CHECK( imaxabs( total - total_ref ) <= total_tol, XLAL_EFUNC, "\nERROR: |total - total_ref| = |%" LAL_UINT8_FORMAT " - %" LAL_UINT8_FORMAT "| > %i", total, total_ref, total_tol );
+  XLAL_CHECK( imaxabs( total - total_ref ) <= total_tol, XLAL_EFUNC, "|total - total_ref| = |%" LAL_UINT8_FORMAT " - %" LAL_UINT8_FORMAT "| > %i", total, total_ref, total_tol );
 
   // Get all points
   gsl_matrix *GAMAT( points, n, total );
@@ -120,7 +120,7 @@ static int SerialisationTest(
     gsl_vector_const_view points_k_view = gsl_matrix_const_column( points, k );
     gsl_vector_sub( point, &points_k_view.vector );
     double err = gsl_blas_dasum( point ) / n;
-    XLAL_CHECK( err < 1e-6, XLAL_EFAILED, "\nERROR: err = %e < 1e-6", err );
+    XLAL_CHECK( err < 1e-6, XLAL_EFAILED, "err = %e < 1e-6", err );
 
     // Checkpoint iterator at certain intervals
     if ( k_ckpt < XLAL_NUM_ELEM( total_ckpt ) && k + 1 >= total_ckpt[k_ckpt] ) {
@@ -174,8 +174,7 @@ static int BasicTest(
   const int bound_on_1,
   const int bound_on_2,
   const int bound_on_3,
-  const UINT4 padding,
-  const char *lattice_name,
+  const TilingLattice lattice,
   const UINT8 total_ref_0,
   const UINT8 total_ref_1,
   const UINT8 total_ref_2,
@@ -184,6 +183,7 @@ static int BasicTest(
 {
 
   const int total_tol = 1;
+  const double value_tol = 1000 * LAL_REAL8_EPS;
 
   const int bound_on[4] = {bound_on_0, bound_on_1, bound_on_2, bound_on_3};
   const UINT8 total_ref[4] = {total_ref_0, total_ref_1, total_ref_2, total_ref_3};
@@ -198,10 +198,6 @@ static int BasicTest(
     XLAL_CHECK( XLALSetLatticeTilingConstantBound( tiling, i, 0.0, bound_on[i] * pow( 100.0, 1.0/n ) ) == XLAL_SUCCESS, XLAL_EFUNC );
   }
 
-  // Set padding level
-  printf( "Padding level: %u\n", padding );
-  XLAL_CHECK( XLALSetLatticeTilingPadding( tiling, padding ) == XLAL_SUCCESS, XLAL_EFUNC );
-
   // Set metric to the Lehmer matrix
   const double max_mismatch = 0.3;
   {
@@ -212,11 +208,24 @@ static int BasicTest(
         gsl_matrix_set( metric, i, j, jj >= ii ? ii/jj : jj/ii );
       }
     }
-    XLAL_CHECK( XLALSetTilingLatticeAndMetric( tiling, lattice_name, metric, max_mismatch ) == XLAL_SUCCESS, XLAL_EFUNC );
+    XLAL_CHECK( XLALSetTilingLatticeAndMetric( tiling, lattice, metric, max_mismatch ) == XLAL_SUCCESS, XLAL_EFUNC );
     GFMAT( metric );
     printf( "Number of (tiled) dimensions: %zu (%zu)\n", XLALTotalLatticeTilingDimensions( tiling ), XLALTiledLatticeTilingDimensions( tiling ) );
     printf( "  Bounds: %i %i %i %i\n", bound_on_0, bound_on_1, bound_on_2, bound_on_3 );
-    printf( "  Lattice type: %s\n", lattice_name );
+    printf( "  Lattice type: %i\n", lattice );
+  }
+
+  // Check tiled status of lattce tiling dimensions
+  for ( size_t i = 0, ti = 0; i < n; ++i ) {
+    const int is_tiled_i = XLALIsTiledLatticeTilingDimension( tiling, i );
+    XLAL_CHECK( is_tiled_i >= 0, XLAL_EFUNC );
+    XLAL_CHECK( !is_tiled_i == !bound_on[i], XLAL_EFAILED, "XLALIsTiledLatticeTilingDimension(tiling, %zu) = %i, should be %i", i, is_tiled_i, bound_on[i] );
+    if ( is_tiled_i ) {
+      const size_t j = XLALLatticeTilingTiledDimension( tiling, ti );
+      XLAL_CHECK( xlalErrno == 0, XLAL_EFUNC );
+      XLAL_CHECK( i == j, XLAL_EFAILED, "XLALLatticeTilingTiledDimension( tiling, %zu ) = %zu, should be %zu", ti, j, i );
+      ++ti;
+    }
   }
 
   // Create lattice tiling locator
@@ -237,12 +246,10 @@ static int BasicTest(
     const UINT8 total = XLALTotalLatticeTilingPoints( itr );
     XLAL_CHECK( total > 0, XLAL_EFUNC );
     printf( "Number of lattice points in %zu dimensions: %" LAL_UINT8_FORMAT " (vs %" LAL_UINT8_FORMAT ", tolerance = %i)\n", i+1, total, total_ref[i], total_tol );
-    XLAL_CHECK( imaxabs( total - total_ref[i] ) <= total_tol, XLAL_EFUNC,
-                "ERROR: |total - total_ref[%zu]| = |%" LAL_UINT8_FORMAT " - %" LAL_UINT8_FORMAT "| > %i", i, total, total_ref[i], total_tol );
+    XLAL_CHECK( imaxabs( total - total_ref[i] ) <= total_tol, XLAL_EFUNC, "|total - total_ref[%zu]| = |%" LAL_UINT8_FORMAT " - %" LAL_UINT8_FORMAT "| > %i", i, total, total_ref[i], total_tol );
     for ( UINT8 k = 0; XLALNextLatticeTilingPoint( itr, NULL ) > 0; ++k ) {
       const UINT8 itr_index = XLALCurrentLatticeTilingIndex( itr );
-      XLAL_CHECK( k == itr_index, XLAL_EFUNC,
-                  "ERROR: k = %" LAL_UINT8_FORMAT " != %" LAL_UINT8_FORMAT " = itr_index", k, itr_index );
+      XLAL_CHECK( k == itr_index, XLAL_EFUNC, "k = %" LAL_UINT8_FORMAT " != %" LAL_UINT8_FORMAT " = itr_index", k, itr_index );
     }
     XLAL_CHECK( XLALResetLatticeTilingIterator( itr ) == XLAL_SUCCESS, XLAL_EFUNC );
 
@@ -252,12 +259,9 @@ static int BasicTest(
       const LatticeTilingStats *stats = XLALLatticeTilingStatistics( tiling, j );
       XLAL_CHECK( stats != NULL, XLAL_EFUNC );
       XLAL_CHECK( stats->name != NULL, XLAL_EFUNC );
-      XLAL_CHECK( imaxabs( stats->total_points - total_ref[j] ) <= total_tol, XLAL_EFAILED, "\n  "
-                  "ERROR: |total - total_ref[%zu]| = |%" LAL_UINT8_FORMAT " - %" LAL_UINT8_FORMAT "| > %i", j, stats->total_points, total_ref[j], total_tol );
-      XLAL_CHECK( stats->min_points <= stats->max_points, XLAL_EFAILED, "\n  "
-                  "ERROR: min_points = %" LAL_INT4_FORMAT " > %" LAL_INT4_FORMAT " = max_points", stats->min_points, stats->max_points );
-      XLAL_CHECK( stats->min_value <= stats->max_value, XLAL_EFAILED, "\n  "
-                  "ERROR: min_value = %g > %g = max_value", stats->min_value, stats->max_value );
+      XLAL_CHECK( imaxabs( stats->total_points - total_ref[j] ) <= total_tol, XLAL_EFAILED, "|total - total_ref[%zu]| = |%" LAL_UINT8_FORMAT " - %" LAL_UINT8_FORMAT "| > %i", j, stats->total_points, total_ref[j], total_tol );
+      XLAL_CHECK( stats->min_points <= stats->max_points, XLAL_EFAILED, "min_points = %" LAL_INT4_FORMAT " > %" LAL_INT4_FORMAT " = max_points", stats->min_points, stats->max_points );
+      XLAL_CHECK( stats->min_value <= stats->max_value, XLAL_EFAILED, "min_value = %g > %g = max_value", stats->min_value, stats->max_value );
       printf( " %s ...", stats->name );
     }
     printf( " done\n" );
@@ -266,6 +270,16 @@ static int BasicTest(
     gsl_matrix *GAMAT( points, n, total );
     XLAL_CHECK( XLALNextLatticeTilingPoints( itr, &points ) == ( int )total, XLAL_EFUNC );
     XLAL_CHECK( XLALNextLatticeTilingPoint( itr, NULL ) == 0, XLAL_EFUNC );
+    for ( UINT8 k = 0; k < total; ++k ) {
+      gsl_vector_const_view point_view = gsl_matrix_const_column( points, k );
+      const gsl_vector *point = &point_view.vector;
+      for ( size_t j = 0; j < n; ++j ) {
+        const double point_j = gsl_vector_get( point, j );
+        const LatticeTilingStats *stats = XLALLatticeTilingStatistics( tiling, j );
+        XLAL_CHECK( point_j >= stats->min_value - value_tol, XLAL_EFAILED, "point_j = %.10g < %.10g = stats[%zu]->min_value", point_j, stats->min_value, j );
+        XLAL_CHECK( point_j <= stats->max_value + value_tol, XLAL_EFAILED, "point_j = %.10g > %.10g = stats[%zu]->max_value", point_j, stats->max_value, j );
+      }
+    }
 
     // Get nearest points to each template, check for consistency
     printf( "  Testing XLALNearestLatticeTiling{Point|Block}() ..." );
@@ -276,37 +290,35 @@ static int BasicTest(
       gsl_vector_const_view point_view = gsl_matrix_const_column( points, k );
       const gsl_vector *point = &point_view.vector;
       XLAL_CHECK( XLALNearestLatticeTilingPoint( loc, point, nearest, nearest_indexes ) == XLAL_SUCCESS, XLAL_EFUNC );
+      for ( size_t j = 0; j < n; ++j ) {
+        const double nearest_j = gsl_vector_get( nearest, j );
+        const LatticeTilingStats *stats = XLALLatticeTilingStatistics( tiling, j );
+        XLAL_CHECK( nearest_j >= stats->min_value - value_tol, XLAL_EFAILED, "nearest_j = %.10g < %.10g = stats[%zu]->min_value", nearest_j, stats->min_value, j );
+        XLAL_CHECK( nearest_j <= stats->max_value + value_tol, XLAL_EFAILED, "nearest_j = %.10g > %.10g = stats[%zu]->max_value", nearest_j, stats->max_value, j );
+      }
       gsl_vector_sub( nearest, point );
       double err = gsl_blas_dasum( nearest ) / n;
-      XLAL_CHECK( err < 1e-6, XLAL_EFAILED, "\n  "
-                  "ERROR: err = %e < 1e-6", err );
-      XLAL_CHECK( nearest_indexes->data[i] == k, XLAL_EFAILED, "\n  "
-                  "ERROR: nearest_indexes[%zu] = %" LAL_UINT8_FORMAT " != %" LAL_UINT8_FORMAT "\n", i, nearest_indexes->data[i], k );
+      XLAL_CHECK( err < 1e-6, XLAL_EFAILED, "err = %e < 1e-6", err );
+      XLAL_CHECK( nearest_indexes->data[i] == k, XLAL_EFAILED, "nearest_indexes[%zu] = %" LAL_UINT8_FORMAT " != %" LAL_UINT8_FORMAT "\n", i, nearest_indexes->data[i], k );
       if ( 0 < i ) {
         const LatticeTilingStats *stats = XLALLatticeTilingStatistics( tiling, i );
         UINT8 nearest_index = 0;
         INT4 nearest_left = 0, nearest_right = 0;
         XLAL_CHECK( XLALNearestLatticeTilingBlock( loc, point, i, nearest, &nearest_index, &nearest_left, &nearest_right ) == XLAL_SUCCESS, XLAL_EFUNC );
-        XLAL_CHECK( nearest_index == nearest_indexes->data[i-1], XLAL_EFAILED, "\n  "
-                    "ERROR: nearest_index = %" LAL_UINT8_FORMAT " != %" LAL_UINT8_FORMAT "\n", nearest_index, nearest_indexes->data[i-1] );
-        XLAL_CHECK( nearest_left <= nearest_right, XLAL_EFAILED, "\n  "
-                    "ERROR: invalid [nearest_left, nearest_right] = [%i, %i]\n", nearest_left, nearest_right );
+        XLAL_CHECK( nearest_index == nearest_indexes->data[i-1], XLAL_EFAILED, "nearest_index = %" LAL_UINT8_FORMAT " != %" LAL_UINT8_FORMAT "\n", nearest_index, nearest_indexes->data[i-1] );
+        XLAL_CHECK( nearest_left <= nearest_right, XLAL_EFAILED, "invalid [nearest_left, nearest_right] = [%i, %i]\n", nearest_left, nearest_right );
         UINT4 nearest_len = nearest_right - nearest_left + 1;
-        XLAL_CHECK( nearest_len <= stats->max_points, XLAL_EFAILED, "\n  "
-                    "ERROR: nearest_len = %i > %i = stats[%zu]->max_points\n", nearest_len, stats->max_points, i );
+        XLAL_CHECK( nearest_len <= stats->max_points, XLAL_EFAILED, "nearest_len = %i > %i = stats[%zu]->max_points\n", nearest_len, stats->max_points, i );
       }
       if ( i+1 < n ) {
         const LatticeTilingStats *stats = XLALLatticeTilingStatistics( tiling, i+1 );
         UINT8 nearest_index = 0;
         INT4 nearest_left = 0, nearest_right = 0;
         XLAL_CHECK( XLALNearestLatticeTilingBlock( loc, point, i+1, nearest, &nearest_index, &nearest_left, &nearest_right ) == XLAL_SUCCESS, XLAL_EFUNC );
-        XLAL_CHECK( nearest_index == nearest_indexes->data[i], XLAL_EFAILED, "\n  "
-                    "ERROR: nearest_index = %" LAL_UINT8_FORMAT " != %" LAL_UINT8_FORMAT "\n", nearest_index, nearest_indexes->data[i] );
-        XLAL_CHECK( nearest_left <= nearest_right, XLAL_EFAILED, "\n  "
-                    "ERROR: invalid [nearest_left, nearest_right] = [%i, %i]\n", nearest_left, nearest_right );
+        XLAL_CHECK( nearest_index == nearest_indexes->data[i], XLAL_EFAILED, "nearest_index = %" LAL_UINT8_FORMAT " != %" LAL_UINT8_FORMAT "\n", nearest_index, nearest_indexes->data[i] );
+        XLAL_CHECK( nearest_left <= nearest_right, XLAL_EFAILED, "invalid [nearest_left, nearest_right] = [%i, %i]\n", nearest_left, nearest_right );
         UINT4 nearest_len = nearest_right - nearest_left + 1;
-        XLAL_CHECK( nearest_len <= stats->max_points, XLAL_EFAILED, "\n  "
-                    "ERROR: nearest_len = %i > %i = stats[%zu]->max_points\n", nearest_len, stats->max_points, i+1 );
+        XLAL_CHECK( nearest_len <= stats->max_points, XLAL_EFAILED, "nearest_len = %i > %i = stats[%zu]->max_points\n", nearest_len, stats->max_points, i+1 );
       }
     }
     printf( " done\n" );
@@ -328,7 +340,7 @@ static int BasicTest(
     while ( XLALNextLatticeTilingPoint( itr_alt, NULL ) > 0 ) {
       ++total_alt;
     }
-    XLAL_CHECK( imaxabs( total_alt - total_ref[i] ) <= total_tol, XLAL_EFUNC, "ERROR: alternating |total - total_ref[%zu]| = |%" LAL_UINT8_FORMAT " - %" LAL_UINT8_FORMAT "| > %i", i, total_alt, total_ref[i], total_tol );
+    XLAL_CHECK( imaxabs( total_alt - total_ref[i] ) <= total_tol, XLAL_EFUNC, "alternating |total - total_ref[%zu]| = |%" LAL_UINT8_FORMAT " - %" LAL_UINT8_FORMAT "| > %i", i, total_alt, total_ref[i], total_tol );
     printf( " done\n" );
 
     // Cleanup
@@ -353,7 +365,7 @@ static int MismatchTest(
   const LatticeTiling *tiling,
   const gsl_matrix *metric,
   const double max_mismatch,
-  const size_t injs_per_point,
+  const double injs_per_point,
   const double mism_hist_error_tol,
   const double mism_out_of_range_tol,
   const UINT8 total_ref,
@@ -374,32 +386,45 @@ static int MismatchTest(
   const UINT8 total = XLALTotalLatticeTilingPoints( itr );
   XLAL_CHECK( total > 0, XLAL_EFUNC );
   printf( "Number of lattice points: %" LAL_UINT8_FORMAT " (vs %" LAL_UINT8_FORMAT ", tolerance = %i)\n", total, total_ref, total_tol );
-  XLAL_CHECK( imaxabs( total - total_ref ) <= total_tol, XLAL_EFUNC, "ERROR: |total - total_ref| = |%" LAL_UINT8_FORMAT " - %" LAL_UINT8_FORMAT "| > %i", total, total_ref, total_tol );
-
-  // Get all points
-  gsl_matrix *GAMAT( points, n, total );
-  XLAL_CHECK( XLALNextLatticeTilingPoints( itr, &points ) == ( int )total, XLAL_EFUNC );
-  XLAL_CHECK( XLALNextLatticeTilingPoint( itr, NULL ) == 0, XLAL_EFUNC );
+  XLAL_CHECK( imaxabs( total - total_ref ) <= total_tol, XLAL_EFUNC, "|total - total_ref| = |%" LAL_UINT8_FORMAT " - %" LAL_UINT8_FORMAT "| > %i", total, total_ref, total_tol );
 
   // Initialise mismatch histogram counts
   double mism_hist[MISM_HIST_BINS] = {0};
   double mism_hist_total = 0, mism_hist_out_of_range = 0;
 
   // Perform 'injs_per_point' injections for every template
-  printf( "Injections per point: %zu\n", injs_per_point );
+  printf( "Injections per point: %g\n", injs_per_point );
   {
-    gsl_matrix *GAMAT( injections, n, total );
-    gsl_matrix *GAMAT( nearest, n, total );
-    gsl_matrix *GAMAT( temp, n, total );
+    UINT8 total_injs = llround(total * injs_per_point);
+    const size_t max_injs_per_batch = 100000;
+
+    // Allocate memory
+    gsl_matrix *GAMAT( max_injections, n, max_injs_per_batch );
+    gsl_matrix *max_nearest = NULL;
+    gsl_matrix *GAMAT( max_temp, n, max_injs_per_batch );
+
+    // Allocate random number generator
     RandomParams *rng = XLALCreateRandomParams( total );
     XLAL_CHECK( rng != NULL, XLAL_EFUNC );
-    for ( size_t i = 0; i < injs_per_point; ++i ) {
+
+    while ( total_injs > 0 ) {
+      const size_t injs_per_batch = GSL_MIN( max_injs_per_batch, total_injs );
+      total_injs -= injs_per_batch;
+      printf("  Injections remaining: %" LAL_UINT8_FORMAT "...\n", total_injs);
+
+      // Create matrix views
+      gsl_matrix_view injections_view = gsl_matrix_submatrix( max_injections, 0, 0, n, injs_per_batch );
+      gsl_matrix *const injections = &injections_view.matrix;
+      gsl_matrix_view temp_view = gsl_matrix_submatrix( max_temp, 0, 0, n, injs_per_batch );
+      gsl_matrix *const temp = &temp_view.matrix;
 
       // Generate random injection points
       XLAL_CHECK( XLALRandomLatticeTilingPoints( tiling, 0.0, rng, injections ) == XLAL_SUCCESS, XLAL_EFUNC );
 
       // Find nearest lattice template points
-      XLAL_CHECK( XLALNearestLatticeTilingPoints( loc, injections, &nearest, NULL ) == XLAL_SUCCESS, XLAL_EFUNC );
+      XLAL_CHECK( XLALNearestLatticeTilingPoints( loc, injections, &max_nearest, NULL ) == XLAL_SUCCESS, XLAL_EFUNC );
+      gsl_matrix_view nearest_view = gsl_matrix_submatrix( max_nearest, 0, 0, n, injs_per_batch );
+      gsl_matrix *const nearest = &nearest_view.matrix;
 
       // Compute mismatch between injections
       gsl_matrix_sub( nearest, injections );
@@ -424,8 +449,8 @@ static int MismatchTest(
     }
 
     // Cleanup
-    GFMAT( injections, nearest, temp );
     XLALDestroyRandomParams( rng );
+    GFMAT( max_injections, max_nearest, max_temp );
 
   }
 
@@ -454,20 +479,20 @@ static int MismatchTest(
   mism_hist_error /= MISM_HIST_BINS;
   printf( "Mismatch histogram error: %0.3e (tolerance %0.3e)\n", mism_hist_error, mism_hist_error_tol );
   if ( mism_hist_error >= mism_hist_error_tol ) {
-    XLAL_ERROR( XLAL_EFAILED, "ERROR: mismatch histogram error exceeds tolerance\n" );
+    XLAL_ERROR( XLAL_EFAILED, "mismatch histogram error exceeds tolerance\n" );
   }
 
   // Check fraction of injections out of histogram range
   const double mism_out_of_range = mism_hist_out_of_range / mism_hist_total;
   printf( "Fraction of points out of histogram range: %0.3e (tolerance %0.3e)\n", mism_out_of_range, mism_out_of_range_tol );
   if ( mism_out_of_range > mism_out_of_range_tol ) {
-    XLAL_ERROR( XLAL_EFAILED, "ERROR: fraction of points out of histogram range exceeds tolerance\n" );
+    XLAL_ERROR( XLAL_EFAILED, "fraction of points out of histogram range exceeds tolerance\n" );
   }
 
   // Perform 10 injections outside parameter space
   {
     gsl_matrix *GAMAT( injections, n, 10 );
-    gsl_matrix *GAMAT( nearest, n, total );
+    gsl_matrix *GAMAT( nearest, n, 10 );
     RandomParams *rng = XLALCreateRandomParams( total );
     XLAL_CHECK( rng != NULL, XLAL_EFUNC );
 
@@ -480,20 +505,18 @@ static int MismatchTest(
     // Cleanup
     GFMAT( injections, nearest );
     XLALDestroyRandomParams( rng );
-
   }
 
   // Cleanup
   XLALDestroyLatticeTilingIterator( itr );
   XLALDestroyLatticeTilingLocator( loc );
-  GFMAT( points );
 
   return XLAL_SUCCESS;
 
 }
 
 static int MismatchSquareTest(
-  const char *lattice_name,
+  const TilingLattice lattice,
   const double freqband,
   const double f1dotband,
   const double f2dotband,
@@ -528,8 +551,8 @@ static int MismatchSquareTest(
       gsl_matrix_set( metric, j, i, gsl_matrix_get( metric, i, j ) );
     }
   }
-  printf( "Lattice type: %s\n", lattice_name );
-  XLAL_CHECK( XLALSetTilingLatticeAndMetric( tiling, lattice_name, metric, max_mismatch ) == XLAL_SUCCESS, XLAL_EFUNC );
+  printf( "Lattice type: %i\n", lattice );
+  XLAL_CHECK( XLALSetTilingLatticeAndMetric( tiling, lattice, metric, max_mismatch ) == XLAL_SUCCESS, XLAL_EFUNC );
 
   // Perform mismatch test
   XLAL_CHECK( MismatchTest( tiling, metric, max_mismatch, 10, 5e-2, 2e-3, total_ref, total_tol, mism_hist_ref ) == XLAL_SUCCESS, XLAL_EFUNC );
@@ -548,7 +571,7 @@ static int MismatchSquareTest(
 }
 
 static int MismatchAgeBrakeTest(
-  const char *lattice_name,
+  const TilingLattice lattice,
   const double freq,
   const double freqband,
   const UINT8 total_ref,
@@ -580,8 +603,8 @@ static int MismatchAgeBrakeTest(
       gsl_matrix_set( metric, j, i, gsl_matrix_get( metric, i, j ) );
     }
   }
-  printf( "Lattice type: %s\n", lattice_name );
-  XLAL_CHECK( XLALSetTilingLatticeAndMetric( tiling, lattice_name, metric, max_mismatch ) == XLAL_SUCCESS, XLAL_EFUNC );
+  printf( "Lattice type: %i\n", lattice );
+  XLAL_CHECK( XLALSetTilingLatticeAndMetric( tiling, lattice, metric, max_mismatch ) == XLAL_SUCCESS, XLAL_EFUNC );
 
   // Perform mismatch test
   XLAL_CHECK( MismatchTest( tiling, metric, max_mismatch, 10, 5e-2, 2e-3, total_ref, total_tol, mism_hist_ref ) == XLAL_SUCCESS, XLAL_EFUNC );
@@ -607,8 +630,6 @@ static int SuperskyTests(
   )
 {
 
-  const int total_tol = 15;
-
   const UINT8 coh_total_ref[3] = {coh_total_ref_0, coh_total_ref_1, coh_total_ref_2};
 
   printf( "Performing super-sky metric tests ...\n\n" );
@@ -623,8 +644,8 @@ static int SuperskyTests(
     LALSeg segment;
     {
       LIGOTimeGPS start_time = ref_time, end_time = ref_time;
-      XLALGPSAdd( &start_time, -3 * Tspan );
-      XLALGPSAdd( &end_time, -2 * Tspan );
+      XLALGPSAdd( &start_time, -100 * Tspan );
+      XLALGPSAdd( &end_time, -98 * Tspan );
       XLAL_CHECK( XLALSegSet( &segment, &start_time, &end_time, 0 ) == XLAL_SUCCESS, XLAL_EFUNC );
       XLAL_CHECK( XLALSegListAppend( &segments, &segment ) == XLAL_SUCCESS, XLAL_EFUNC );
     }
@@ -637,8 +658,8 @@ static int SuperskyTests(
     }
     {
       LIGOTimeGPS start_time = ref_time, end_time = ref_time;
-      XLALGPSAdd( &start_time, 2.5 * Tspan );
-      XLALGPSAdd( &end_time, 3.5 * Tspan );
+      XLALGPSAdd( &start_time, 92.5 * Tspan );
+      XLALGPSAdd( &end_time, 93.5 * Tspan );
       XLAL_CHECK( XLALSegSet( &segment, &start_time, &end_time, 0 ) == XLAL_SUCCESS, XLAL_EFUNC );
       XLAL_CHECK( XLALSegListAppend( &segments, &segment ) == XLAL_SUCCESS, XLAL_EFUNC );
     }
@@ -648,11 +669,11 @@ static int SuperskyTests(
     .length = 1,
     .sites = { lalCachedDetectors[LAL_LLO_4K_DETECTOR] }
   };
-  EphemerisData *edat = XLALInitBarycenter( TEST_DATA_DIR "earth00-19-DE405.dat.gz",
-                                            TEST_DATA_DIR "sun00-19-DE405.dat.gz" );
+  EphemerisData *edat = XLALInitBarycenter( TEST_PKG_DATA_DIR "earth00-19-DE405.dat.gz",
+                                            TEST_PKG_DATA_DIR "sun00-19-DE405.dat.gz" );
   XLAL_CHECK( edat != NULL, XLAL_EFUNC );
   const double freq_max = 40.0;
-  SuperskyMetrics *metrics = XLALComputeSuperskyMetrics( 1, &ref_time, &segments, freq_max, &detectors, NULL, DETMOTION_SPIN | DETMOTION_PTOLEORBIT, edat );
+  SuperskyMetrics *metrics = XLALComputeSuperskyMetrics( SUPERSKY_METRIC_TYPE, 1, &ref_time, &segments, freq_max, &detectors, NULL, DETMOTION_SPIN | DETMOTION_PTOLEORBIT, edat );
   XLAL_CHECK( metrics != NULL, XLAL_EFUNC );
   XLAL_CHECK( metrics->num_segments == segments.length, XLAL_EFAILED );
 
@@ -670,9 +691,8 @@ static int SuperskyTests(
   XLAL_CHECK( semi_tiling != NULL, XLAL_EFUNC );
 
   // Add bounds
-  double alpha1 = 0, alpha2 = 0, delta1 = 0, delta2 = 0;
-  const double freq_min = freq_max - 5e-5, f1dot = -5e-9;
-  XLAL_CHECK( XLALComputePhysicalSkyEqualAreaPatch( &alpha1, &alpha2, &delta1, &delta2, 2, 0 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  const double alpha1 = 0, alpha2 = LAL_PI, delta1 = -LAL_PI_2, delta2 = LAL_PI_2;
+  const double freq_min = freq_max - 0.05, f1dot = -3e-9;
   for ( size_t n = 0; n < metrics->num_segments; ++n ) {
     XLAL_CHECK( XLALSetSuperskyPhysicalSkyBounds( coh_tiling[n], metrics->coh_rssky_metric[n], metrics->coh_rssky_transf[n], alpha1, alpha2, delta1, delta2 ) == XLAL_SUCCESS, XLAL_EFUNC );
     XLAL_CHECK( XLALSetSuperskyPhysicalSpinBound( coh_tiling[n], metrics->coh_rssky_transf[n], 0, freq_min, freq_max ) == XLAL_SUCCESS, XLAL_EFUNC );
@@ -684,18 +704,17 @@ static int SuperskyTests(
 
   // Set metric
   for ( size_t n = 0; n < metrics->num_segments; ++n ) {
-    XLAL_CHECK( XLALSetTilingLatticeAndMetric( coh_tiling[n], "Ans", metrics->coh_rssky_metric[n], coh_max_mismatch ) == XLAL_SUCCESS, XLAL_EFUNC );
+    XLAL_CHECK( XLALSetTilingLatticeAndMetric( coh_tiling[n], TILING_LATTICE_ANSTAR, metrics->coh_rssky_metric[n], coh_max_mismatch ) == XLAL_SUCCESS, XLAL_EFUNC );
   }
-  XLAL_CHECK( XLALSetTilingLatticeAndMetric( semi_tiling, "Ans", metrics->semi_rssky_metric, semi_max_mismatch ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK( XLALSetTilingLatticeAndMetric( semi_tiling, TILING_LATTICE_ANSTAR, metrics->semi_rssky_metric, semi_max_mismatch ) == XLAL_SUCCESS, XLAL_EFUNC );
 
   // Check lattice step sizes in frequency
   const size_t ifreq = 3;
-  const double semi_dfreq = XLALLatticeTilingStepSizes( semi_tiling, ifreq );
+  const double semi_dfreq = XLALLatticeTilingStepSize( semi_tiling, ifreq );
   for ( size_t n = 0; n < metrics->num_segments; ++n ) {
-    const double coh_dfreq = XLALLatticeTilingStepSizes( coh_tiling[n], ifreq );
+    const double coh_dfreq = XLALLatticeTilingStepSize( coh_tiling[n], ifreq );
     const double tol = 1e-8;
-    XLAL_CHECK( fabs( coh_dfreq - semi_dfreq ) < tol * semi_dfreq, XLAL_EFAILED,
-                "  ERROR: semi_dfreq=%0.15e, coh_dfreq[%zu]=%0.15e, |coh_dfreq - semi_dfreq| >= %0.5g * semi_dfreq", semi_dfreq, n, coh_dfreq, tol );
+    XLAL_CHECK( fabs( coh_dfreq - semi_dfreq ) < tol * semi_dfreq, XLAL_EFAILED, "semi_dfreq=%0.15e, coh_dfreq[%zu]=%0.15e, |coh_dfreq - semi_dfreq| >= %0.5g * semi_dfreq", semi_dfreq, n, coh_dfreq, tol );
   }
 
   // Print information on bounds
@@ -704,6 +723,8 @@ static int SuperskyTests(
       const LatticeTilingStats *stats = XLALLatticeTilingStatistics( coh_tiling[n], i );
       XLAL_CHECK( stats != NULL, XLAL_EFUNC );
       XLAL_CHECK( stats->name != NULL, XLAL_EFUNC );
+      const int j = XLALLatticeTilingDimensionByName( coh_tiling[n], stats->name );
+      XLAL_CHECK( j == (int)i, XLAL_EFUNC, "XLALLatticeTilingDimensionByName(..., \"%s\") = %i != %zu", stats->name, j, i );
       printf( "Coherent #%zu  bound #%zu: name=%6s, points=[%4u,%4u]\n", n, i, stats->name, stats->min_points, stats->max_points );
     }
   }
@@ -711,26 +732,20 @@ static int SuperskyTests(
     const LatticeTilingStats *stats = XLALLatticeTilingStatistics( semi_tiling, i );
     XLAL_CHECK( stats != NULL, XLAL_EFUNC );
     XLAL_CHECK( stats->name != NULL, XLAL_EFUNC );
+    const int j = XLALLatticeTilingDimensionByName( semi_tiling, stats->name );
+    XLAL_CHECK( j == (int)i, XLAL_EFUNC, "XLALLatticeTilingDimensionByName(..., \"%s\") = %i != %zu", stats->name, j, i );
     printf( "Semicoherent bound #%zu: name=%6s, points=[%4u,%4u]\n", i, stats->name, stats->min_points, stats->max_points );
   }
   printf( "\n" );
 
-  // Check computation of spindown range for coherent tilings
-  for ( size_t n = 0; n < metrics->num_segments; ++n ) {
-    PulsarSpinRange spin_range;
-    XLAL_CHECK( XLALSuperskyLatticePulsarSpinRange( &spin_range, coh_tiling[n], metrics->coh_rssky_transf[n] ) == XLAL_SUCCESS, XLAL_EFUNC );
-    printf( "Coherent #%zu spindown range: freq=[%0.5g,%0.5g], f1dot=[%0.5g,%0.5g]\n", n, spin_range.fkdot[0], spin_range.fkdot[0] + spin_range.fkdotBand[0], spin_range.fkdot[1], spin_range.fkdot[1] + spin_range.fkdotBand[1] );
-  }
-  printf( "\n" );
-
   // Perform mismatch test of coherent and semicoherent tilings
   for ( size_t n = 0; n < metrics->num_segments; ++n ) {
     printf( "Coherent #%zu mismatch tests:\n", n );
-    XLAL_CHECK( MismatchTest( coh_tiling[n], metrics->coh_rssky_metric[n], coh_max_mismatch, 1, 5e-2, 4e-3, coh_total_ref[n], total_tol, A4s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+    XLAL_CHECK( MismatchTest( coh_tiling[n], metrics->coh_rssky_metric[n], coh_max_mismatch, 0.1, 5e-2, 5e-2, coh_total_ref[n], lround(1e-5 * coh_total_ref[n]), A4s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
     printf( "\n" );
   }
   printf( "Semicoherent mismatch tests:\n" );
-  XLAL_CHECK( MismatchTest( semi_tiling, metrics->semi_rssky_metric, semi_max_mismatch, 1, 5e-2, 1e-2, semi_total_ref, total_tol, A4s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK( MismatchTest( semi_tiling, metrics->semi_rssky_metric, semi_max_mismatch, 0.0001, 5e-2, 5e-2, semi_total_ref, lround(1e-6 * semi_total_ref), A4s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
   printf( "\n" );
 
   // Cleanup
@@ -757,51 +772,54 @@ int main( void )
   setvbuf( stderr, NULL, _IONBF, 0 );
 
   // Perform basic tests
-  XLAL_CHECK_MAIN( BasicTest( 1, 0, 0, 0, 0, 1, "Zn" ,    1,    1,    1,    1 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 1, 1, 1, 1, 1, 1, "Ans",   93,    0,    0,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 1, 1, 1, 1, 1, 1, "Zn" ,   93,    0,    0,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 2, 0, 0, 0, 0, 1, "Ans",    1,    1,    1,    1 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 2, 1, 1, 1, 1, 1, "Ans",   12,  144,    0,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 2, 1, 1, 1, 1, 1, "Zn" ,   13,  190,    0,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 3, 0, 0, 0, 0, 1, "Zn" ,    1,    1,    1,    1 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 3, 1, 1, 1, 1, 1, "Ans",    8,   46,  332,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 3, 1, 1, 1, 1, 1, "Zn" ,    8,   60,  583,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 0, 0, 0, 0, 1, "Ans",    1,    1,    1,    1 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 0, 0, 0, 1, 1, "Ans",    1,    1,    1,    4 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 0, 0, 1, 0, 1, "Ans",    1,    1,    4,    4 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 0, 0, 1, 1, 1, "Ans",    1,    1,    4,   20 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 0, 1, 0, 0, 1, "Ans",    1,    4,    4,    4 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 0, 1, 0, 1, 1, "Ans",    1,    5,    5,   25 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 0, 1, 1, 0, 1, "Ans",    1,    5,   24,   24 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 0, 1, 1, 1, 1, "Ans",    1,    5,   20,  115 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 0, 0, 0, 1, "Ans",    4,    4,    4,    4 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 0, 0, 1, 1, "Ans",    5,    5,    5,   23 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 0, 1, 0, 1, "Ans",    5,    5,   23,   23 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 0, 1, 1, 1, "Ans",    6,    6,   24,  139 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 0, 0, 1, "Ans",    5,   25,   25,   25 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 0, 1, 1, "Ans",    6,   30,   30,  162 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 1, 0, 1, "Ans",    6,   27,  151,  151 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 1, 1, 1, "Ans",    6,   30,  145,  897 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 1, 1, 1, "Zn" ,    7,   46,  287, 2543 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 1, 1, 2, "Ans",    8,   54,  336, 2804 ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 1, 1, 2, "Zn" ,    9,   77,  661, 7822 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 1, 0, 0, 0, 0, TILING_LATTICE_CUBIC,     1,    1,    1,    1 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 1, 1, 1, 1, 1, TILING_LATTICE_ANSTAR,   93,    0,    0,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 1, 1, 1, 1, 1, TILING_LATTICE_CUBIC,    93,    0,    0,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 2, 0, 0, 0, 0, TILING_LATTICE_ANSTAR,    1,    1,    1,    1 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 2, 1, 1, 1, 1, TILING_LATTICE_ANSTAR,   12,  144,    0,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 2, 1, 1, 1, 1, TILING_LATTICE_CUBIC,    13,  190,    0,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 3, 0, 0, 0, 0, TILING_LATTICE_CUBIC,     1,    1,    1,    1 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 3, 1, 1, 1, 1, TILING_LATTICE_ANSTAR,    8,   46,  332,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 3, 1, 1, 1, 1, TILING_LATTICE_CUBIC,     8,   60,  583,    0 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 0, 0, 0, 0, TILING_LATTICE_ANSTAR,    1,    1,    1,    1 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 0, 0, 0, 1, TILING_LATTICE_ANSTAR,    1,    1,    1,    4 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 0, 0, 1, 0, TILING_LATTICE_ANSTAR,    1,    1,    4,    4 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 0, 0, 1, 1, TILING_LATTICE_ANSTAR,    1,    1,    4,   20 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 0, 1, 0, 0, TILING_LATTICE_ANSTAR,    1,    4,    4,    4 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 0, 1, 0, 1, TILING_LATTICE_ANSTAR,    1,    5,    5,   25 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 0, 1, 1, 0, TILING_LATTICE_ANSTAR,    1,    5,   24,   24 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 0, 1, 1, 1, TILING_LATTICE_ANSTAR,    1,    5,   20,  115 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 1, 0, 0, 0, TILING_LATTICE_ANSTAR,    4,    4,    4,    4 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 1, 0, 0, 1, TILING_LATTICE_ANSTAR,    5,    5,    5,   23 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 1, 0, 1, 0, TILING_LATTICE_ANSTAR,    5,    5,   23,   23 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 1, 0, 1, 1, TILING_LATTICE_ANSTAR,    6,    6,   24,  139 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 0, 0, TILING_LATTICE_ANSTAR,    5,   25,   25,   25 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 0, 1, TILING_LATTICE_ANSTAR,    6,   30,   30,  162 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 1, 0, TILING_LATTICE_ANSTAR,    6,   27,  151,  151 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 1, 1, TILING_LATTICE_ANSTAR,    6,   30,  145,  897 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( BasicTest( 4, 1, 1, 1, 1, TILING_LATTICE_CUBIC,     7,   46,  287, 2543 ) == XLAL_SUCCESS, XLAL_EFUNC );
 
   // Perform mismatch tests with a square parameter space
-  XLAL_CHECK_MAIN( MismatchSquareTest( "Zn",  0.03,     0,     0, 21460,  Z1_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( MismatchSquareTest( "Zn",  2e-4, -2e-9,     0, 23763,  Z2_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( MismatchSquareTest( "Zn",  1e-4, -1e-9, 1e-17, 19550,  Z3_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( MismatchSquareTest( "Ans", 0.03,     0,     0, 21460, A1s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( MismatchSquareTest( "Ans", 2e-4, -2e-9,     0, 18283, A2s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( MismatchSquareTest( "Ans", 1e-4, -2e-9, 2e-17, 20268, A3s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( MismatchSquareTest( TILING_LATTICE_CUBIC,  0.03,     0,     0, 21460,  Z1_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( MismatchSquareTest( TILING_LATTICE_CUBIC,  2e-4, -2e-9,     0, 23763,  Z2_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( MismatchSquareTest( TILING_LATTICE_CUBIC,  1e-4, -1e-9, 1e-17, 19550,  Z3_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( MismatchSquareTest( TILING_LATTICE_ANSTAR, 0.03,     0,     0, 21460, A1s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( MismatchSquareTest( TILING_LATTICE_ANSTAR, 2e-4, -2e-9,     0, 18283, A2s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( MismatchSquareTest( TILING_LATTICE_ANSTAR, 1e-4, -2e-9, 2e-17, 20268, A3s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
 
   // Perform mismatch tests with an age--braking index parameter space
-  XLAL_CHECK_MAIN( MismatchAgeBrakeTest( "Ans", 100, 4.0e-5, 37872, A3s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( MismatchAgeBrakeTest( "Ans", 200, 1.5e-5, 37232, A3s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
-  XLAL_CHECK_MAIN( MismatchAgeBrakeTest( "Ans", 300, 1.0e-5, 37022, A3s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( MismatchAgeBrakeTest( TILING_LATTICE_ANSTAR, 100, 4.0e-5, 37870, A3s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( MismatchAgeBrakeTest( TILING_LATTICE_ANSTAR, 200, 1.5e-5, 37230, A3s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( MismatchAgeBrakeTest( TILING_LATTICE_ANSTAR, 300, 1.0e-5, 37022, A3s_mism_hist ) == XLAL_SUCCESS, XLAL_EFUNC );
 
   // Perform a variety of tests with the reduced supersky parameter space and metric
-  XLAL_CHECK_MAIN( SuperskyTests( 99376, 80817, 63091, 482182 ) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK_MAIN( SuperskyTests( 6886488, 1050134, 932765, 26063227993 ) == XLAL_SUCCESS, XLAL_EFUNC );
 
   return EXIT_SUCCESS;
 
 }
+
+// Local Variables:
+// c-file-style: "linux"
+// c-basic-offset: 2
+// End:
